import time
import requests
import pandas as pd
import os
from ta.trend import SMAIndicator, EMAIndicator, MACD
from ta.volatility import BollingerBands
from ta.momentum import RSIIndicator
from tqdm import tqdm

# Constants
RAW_HEADERS = ['Timestamp', 'Open', 'High', 'Low', 'Close', 'Volume']
INDICATOR_HEADERS = ['SMA', 'EMA', 'RSI', 'MACD', 'bollinger_mavg', 'bollinger_hband', 'bollinger_lband']
API_URL = 'https://api.crypto.com/v2/public/get-candlestick?instrument_name=ETH_USD&interval=5m'
OUTPUT_FILE = 'ta_data.csv'
SLEEP_TIME = 5 * 60  # Time to sleep between requests, in seconds
INDICATOR_WINDOW = 14
BOLLINGER_WINDOW = 20
BOLLINGER_WINDOW_DEV = 2
MAX_ROWS = 250

# Function to fetch data from the API
def fetch_data(url, params=None):
    response = requests.get(url, params=params)
    response.raise_for_status()  # Raise an HTTPError if the response was an HTTP error
    data = response.json()

    if 'error' in data:
        print(f"Error fetching data: {data['error']}")
        return None

    return data['result']['data']

# Function to process the fetched data and convert it into a Pandas DataFrame
def process_data(data):
    return [{'Timestamp': int(row['t']), 'Open': float(row['o']), 'High': float(row['h']),
             'Low': float(row['l']), 'Close': float(row['c']), 'Volume': float(row['v'])} 
            for row in data]

# Function to calculate the technical analysis indicators on the fetched data
def calculate_indicators(df):
    df['SMA'] = SMAIndicator(df['Close'], window=INDICATOR_WINDOW).sma_indicator()
    df['EMA'] = EMAIndicator(df['Close'], window=INDICATOR_WINDOW).ema_indicator()
    df['RSI'] = RSIIndicator(df['Close'], window=INDICATOR_WINDOW).rsi()
    df['MACD'] = MACD(df['Close']).macd()
    bollinger = BollingerBands(close=df['Close'], window=BOLLINGER_WINDOW, window_dev=BOLLINGER_WINDOW_DEV)
    df['bollinger_mavg'] = bollinger.bollinger_mavg()
    df['bollinger_hband'] = bollinger.bollinger_hband()
    df['bollinger_lband'] = bollinger.bollinger_lband()

# Function to write the dataframe to a CSV file
def write_to_csv(df, file_name, mode='a', header=False):
    df.to_csv(file_name, mode=mode, header=header, index=False)

# Check if the output file exists before writing headers
if not os.path.isfile(OUTPUT_FILE):
    pd.DataFrame(columns=RAW_HEADERS+INDICATOR_HEADERS).to_csv(OUTPUT_FILE, index=False)

# Main loop
while True:
    try:
        print("Requesting data...")
        data = fetch_data(API_URL)

        if data is None:
            time.sleep(SLEEP_TIME)
            continue

        # Process the fetched data
        new_rows = process_data(data)
        df = pd.read_csv(OUTPUT_FILE)
        df = pd.concat([df, pd.DataFrame(new_rows)], ignore_index=True)

        # Limit the number of rows in the dataframe
        df = df.tail(MAX_ROWS)

        print("Calculating indicators...")
        calculate_indicators(df)
        print("Indicators calculated.")

        # Write dataframe to csv
        write_to_csv(df, OUTPUT_FILE, mode='w', header=True)
        print("Data saved to CSV file.")

    except requests.HTTPError as http_err:
        print(f"HTTP error occurred: {http_err}")
    except ValueError as json_err:
        print(f"JSON error occurred: {json_err}")
    except Exception as err:
        print(f"An error occurred: {err}")

    finally:
        print("Sleeping for 5 minutes.")
        # Progress bar for the sleeping period
        for remaining_time in tqdm(range(SLEEP_TIME), desc="Progress"):
            time.sleep(1)
